﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Ink;

namespace SilverlightDraw
{
  public partial class Page : UserControl
  {
    private Stroke _currentStroke = null;
    
    public Page()
    {
      InitializeComponent();

      Color strokeColor = GetColor();
      SettingsButton.Background = new SolidColorBrush(strokeColor);
      (SettingsPopup.Child as Border).Background = SettingsButton.Background;
    }

    private Color GetColor()
    {
      if (ColorRSlider == null
        || ColorGSlider == null
        || ColorBSlider == null)
      {
        return Colors.Black;
      }

      Color strokeColor = Color.FromArgb(255,
        (byte) ColorRSlider.Value,
        (byte) ColorGSlider.Value,
        (byte) ColorBSlider.Value);
      return strokeColor;
    }

    private void InkPresenter_MouseLeftButtonDown(object sender,
      MouseButtonEventArgs e)
    {
      SettingsPopup.IsOpen = false;
      _currentStroke = new Stroke(e.StylusDevice.GetStylusPoints(MyInkPresenter));
      _currentStroke.DrawingAttributes = new DrawingAttributes();
    
      Color strokeColor = GetColor();
      _currentStroke.DrawingAttributes.Color = strokeColor;
      _currentStroke.DrawingAttributes.OutlineColor = strokeColor;
    
      _currentStroke.DrawingAttributes.Height =
        _currentStroke.DrawingAttributes.Width = SizeSlider.Value;
      MyInkPresenter.Strokes.Add(_currentStroke);
    }

    private void InkPresenter_MouseMove(object sender,
      MouseEventArgs e)
    {
      if (_currentStroke != null)
      {
        _currentStroke.StylusPoints.Add(e.StylusDevice.GetStylusPoints(MyInkPresenter));
      }
    }

    private void InkPresenter_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
    {
      _currentStroke = null;
    }
    private void LayoutRoot_MouseLeave(object sender, MouseEventArgs e)
    {
      _currentStroke = null;
    }

    private void SettingsButton_Click(object sender,
      RoutedEventArgs e)
    {
      SettingsPopup.IsOpen = true;
    }

    private void ColorSlider_ValueChanged(object sender,
      RoutedPropertyChangedEventArgs<double> e)
    {
      if (SettingsButton == null || SettingsPopup == null)
      {
        return;
      }
      Color strokeColor = GetColor();
      SettingsButton.Background = new SolidColorBrush(strokeColor);
      (SettingsPopup.Child as Border).Background = SettingsButton.Background;
    }

    private void SizeSlider_ValueChanged(object sender,
      RoutedPropertyChangedEventArgs<double> e)
    {
      if (SettingsPopup == null)
      {
        return;
      }
      Thickness borderSize = new Thickness(SizeSlider.Value);
      (SettingsPopup.Child as Border).BorderThickness = borderSize;
    }
  }
}
