﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Thumbnails.Data;

namespace Thumbnails
{
	public partial class ThumbnailsViewerControl : UserControl
	{
    private static void OnItemsSourceChanged(DependencyObject d,
      DependencyPropertyChangedEventArgs args)
    {
      ThumbnailsViewerControl sender = d as ThumbnailsViewerControl;
      sender.ThumbItemsControl.ItemsSource
        = (args.NewValue as MediaExCollection);
    }

    public static readonly DependencyProperty ItemsSourceProperty
      = DependencyProperty.Register("ItemsSource",
      typeof(MediaExCollection), typeof(ThumbnailsViewerControl), 
      new PropertyMetadata(new PropertyChangedCallback(OnItemsSourceChanged)));

    public MediaExCollection ItemsSource
    {
      get { return (MediaExCollection) GetValue(ItemsSourceProperty); }
      set { SetValue(ItemsSourceProperty, value); }
    }

    public MediaEx SelectedItem
    {
      get { return (MediaEx) GetValue(SelectedItemProperty); }
      set { SetValue(SelectedItemProperty, value); }
    }

    public static readonly DependencyProperty SelectedItemProperty
      = DependencyProperty.Register("SelectedItem",
      typeof(MediaEx), typeof(ThumbnailsViewerControl),
      new PropertyMetadata(null));

    public event SelectionChangedEventHandler SelectionChanged;

    private void OnSelectionChanged(FrameworkElement sender, MediaEx item)
    {
      if (SelectionChanged != null)
      {
        object[] addedItems = new object[1];
        addedItems[0] = sender;
        object[] removedItems = new object[0];
        SelectionChangedEventArgs args
            = new SelectionChangedEventArgs(removedItems, addedItems);
        SelectionChanged(this, args);
      }
      SelectedItem = item;
    }


    
    public ThumbnailsViewerControl()
		{
			// Required to initialize variables
			InitializeComponent();

      ScrollButtonUp.IsEnabled = false;
      ScrollButtonDown.IsEnabled = false;
		}

    private void media_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
    {
      FrameworkElement clickedElement = sender as FrameworkElement;
      MediaEx media = clickedElement.DataContext as MediaEx;
      OnSelectionChanged(clickedElement, media);
    }

    private void MediaElement_MediaEnded(object sender, RoutedEventArgs e)
    {
      (sender as MediaElement).Stop();
      (sender as MediaElement).Play();
    }

    private double _scrollOffset = 0.0;
    private void ScrollButtonUp_Click(object sender, RoutedEventArgs e)
    {
      ScrollButtonDown.IsEnabled = true;
      _scrollOffset -= ThumbScrollViewer.ActualWidth / 10;
      if (_scrollOffset <= 0)
      {
        _scrollOffset = 0;
        ScrollButtonUp.IsEnabled = false;
      }
      ThumbScrollViewer.ScrollToVerticalOffset(_scrollOffset);
    }
    
    private void ScrollButtonDown_Click(object sender, RoutedEventArgs e)
    {
      ScrollButtonUp.IsEnabled = true;
      _scrollOffset += ThumbScrollViewer.ActualHeight / 10;
      if (_scrollOffset >= ThumbScrollViewer.ScrollableHeight)
      {
        _scrollOffset = ThumbScrollViewer.ScrollableHeight;
        ScrollButtonDown.IsEnabled = false;
      }
      ThumbScrollViewer.ScrollToVerticalOffset(_scrollOffset);
    }

    private void ThumbnailsViewerControl_LayoutUpdated(object sender, 
      EventArgs e)
    {
      if (ThumbScrollViewer.ScrollableHeight == 0)
      {
        ScrollButtonUp.IsEnabled = false;
        ScrollButtonDown.IsEnabled = false;
      }
      else
      {
        if (ThumbScrollViewer.VerticalOffset > 0)
        {
          ScrollButtonUp.IsEnabled = true;
        }
        if (ThumbScrollViewer.VerticalOffset
             < ThumbScrollViewer.ScrollableHeight)
        {
          ScrollButtonDown.IsEnabled = true;
        }
      }
      _scrollOffset = ThumbScrollViewer.VerticalOffset;
    }
	}
}