﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Browser;

namespace JavaScriptDotNet
{
  public partial class Page : UserControl
  {
    [ScriptableMember]
    public event CounterTickedHandler CounterTicked;

    public void OnCounterTicked(CounterTickedEventArgs e)
    {
      if (CounterTicked != null)
      {
        CounterTicked(this, e);
      }
    }

    public Page(int initParam1, string initParam2)
    {
      InitializeComponent();
      HtmlPage.RegisterScriptableObject("Page", this);

      HtmlPage.Window.Alert("Retrieved init params:"
        + Environment.NewLine + "initParam1 = " + initParam1
        + Environment.NewLine + "initParam2 = " + initParam2);

      HtmlDocument document = HtmlPage.Document;
      HtmlElement button = document.GetElementById("htmlButton");
      button.AttachEvent("onclick", HtmlButton_Click);
    }

    string _originalButtonValue = null;
    void HtmlButton_Click(object sender, HtmlEventArgs e)
    {
      TickAndCheckCounter();
    
      HtmlElement element = e.Source as HtmlElement;
      if (element != null)
      {
        if (_originalButtonValue == null)
        {
          _originalButtonValue = element.GetAttribute("value");
        }
        element.SetAttribute("value", _originalButtonValue
          + " (" + _counter.ToString() + ")");
      }
    }

    private int _counter = 0;
    private void CounterButton_Click(object sender, RoutedEventArgs e)
    {
      TickAndCheckCounter();
    }

    private void TickAndCheckCounter()
    {
      _counter++;
      if ((_counter % 5) == 0)
      {
        // true every time the counter is a multiple of 5
        CounterTickedEventArgs args = new CounterTickedEventArgs();
        args.CounterValue = _counter;
        MyDate customTimeStamp = new MyDate();
        customTimeStamp.When = DateTime.Now;
        args.CustomTimestamp = customTimeStamp;
        OnCounterTicked(args);
      }
    }

    private void SaySomethingButton_Click(object sender,
      RoutedEventArgs e)
    {
      ScriptObject instance
        = HtmlPage.Window.GetProperty("jsDotNetInstance") as ScriptObject;
      instance.Invoke("saySomething",
        new object[] { "Hello", "World", DateTime.Now });
    }

    private void SetTitleButton_Click(object sender, RoutedEventArgs e)
    {
      HtmlPage.Document.SetProperty("title", ValueTextBox.Text);
    }
    private void SetTopButton_Click(object sender, RoutedEventArgs e)
    {
      HtmlElement div = HtmlPage.Document.GetElementById("TestDiv");
      div.SetStyleAttribute("top", ValueTextBox.Text);
    
      HtmlElement span = HtmlPage.Document.GetElementById("TestSpan");
      span.SetAttribute("innerHTML",
        "<span style='font-style: bold'>"
        + ValueTextBox.Text + "</span>");
    }

    private void SimpleTextBlock_MouseLeftButtonDown(object sender,
      MouseButtonEventArgs e)
    {
      ScriptObject instance = HtmlPage.Window.GetProperty("jsDotNetInstance")
        as ScriptObject;

      int valueInt = (int) (double) instance.Invoke("getMeANumber", null);
      string valueString = (string) instance.Invoke("getMeAString", null);
      bool valueBool = (bool) instance.Invoke("getMeABoolean", null);
      SimpleTextBlock.Text = valueInt.ToString()
        + " - " + valueString + " - " + valueBool.ToString();
    }
    private void ObjectTextBlock_MouseLeftButtonDown(object sender,
      MouseButtonEventArgs e)
    {
      ScriptObject instance = HtmlPage.Window.GetProperty("jsDotNetInstance")
        as ScriptObject;

      ScriptObject value = instance.Invoke("getMeAnObject", null) as ScriptObject;
      ObjectTextBlock.Text = (string) value.GetProperty("Property1")
        + " - " + (int) (double) value.GetProperty("Property2");
    }
    private void ArrayTextBlock_MouseLeftButtonDown(object sender,
      MouseButtonEventArgs e)
    {
      ScriptObject instance = HtmlPage.Window.GetProperty("jsDotNetInstance")
        as ScriptObject;

      ScriptObject value = instance.Invoke("getMeAnArray", null) as ScriptObject;
      int length = (int) (double) value.GetProperty("length");
      ArrayTextBlock.Text = "";
      for (int index = 0; index < length; index++)
      {
        ArrayTextBlock.Text += value.GetProperty(index);
      }
    }

  }

  public class CounterTickedEventArgs : EventArgs
  {
    [ScriptableMember]
    public int CounterValue
    {
      get;
      internal set;
    }

    [ScriptableMember]
    public MyDate CustomTimestamp
    {
      get;
      internal set;
    }
  }

  public delegate void CounterTickedHandler(object sender,
    CounterTickedEventArgs e);

  public class MyDate
  {
    [ScriptableMember]
    public DateTime When
    {
      get;
      internal set;
    }

    public override string ToString()
    {
      // return When.ToString(); // Not formatted
      return When.ToString("R"); // Format RFC1123
    }
  }

}
