﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;
using Thumbnails.Data;
using System.Windows.Browser;
using UserManagement;

namespace Thumbnails
{
	public partial class Page : UserControl
	{
    [ScriptableMember]
    public event EventHandler LoginClicked;

    private FrameworkElement _expandedMedia = null;
    private double _expansionFactor = 1.0;
    private Storyboard _fadeStoryboard = null;
    private DataFile _dataFile;
    private MediaInformationFile _mediaFile;

    public Page()
    {
      // Required to initialize variables
      InitializeComponent();

      Storyboard expandStoryboard
        = TopUserControl.Resources["ExpandStoryboard"] as Storyboard;
      expandStoryboard.Completed
        += new EventHandler(expandStoryboard_Completed);

      _dataFile = new DataFile();
      _dataFile.LoadUsers();

      HtmlPage.RegisterScriptableObject("Page", this);

      ThumbnailsViewer.SelectionChanged
        += new SelectionChangedEventHandler(ThumbnailsViewer_SelectionChanged);

      this.Loaded += new RoutedEventHandler(Page_Loaded);
    }

    void Page_Loaded(object sender, RoutedEventArgs e)
    {
      _mediaFile = new MediaInformationFile();
      _mediaFile.Loaded += new EventHandler(_mediaFile_Loaded);
      _mediaFile.LoadMediaFile();
    }

    void _mediaFile_Loaded(object sender, EventArgs e)
    {
      MediaFileLoadedEventArgs args = e as MediaFileLoadedEventArgs;

      if (args.Collection == null)
      {
        StatusTextBlock.Text = "There was an error";
        if (args.Error != null)
        {
          ErrorTextBlock.Text = args.Error.Message
            + Environment.NewLine + args.Error.StackTrace;
        }
        return;
      }

      MediaExCollection mediaCollection
        = Resources["MediaDataSource"] as MediaExCollection;

      bool foundBrush = false;
      foreach (MediaEx mediaInfo in args.Collection)
      {
        mediaCollection.Add(mediaInfo);

        if (!foundBrush
          && mediaInfo.Type == MediaInfo.MediaType.Image)
        {
          ImageBrush brush = new ImageBrush();
          brush.ImageSource = mediaInfo.ImageSource;
          TitleTextBlock1.Foreground = brush;
          TitleTextBlock2.Foreground = brush;
          foundBrush = true;
        }
      }

      Storyboard cacheFadeOut
        = Resources["CacheFadeOutStoryboard"] as Storyboard;
      cacheFadeOut.Completed
        += new EventHandler(cacheFadeOut_Completed);
      cacheFadeOut.Begin();
    }

    void cacheFadeOut_Completed(object sender, EventArgs e)
    {
      Cache.Visibility = Visibility.Collapsed;
    }

    void ThumbnailsViewer_SelectionChanged(object sender,
      SelectionChangedEventArgs e)
    {
      if (e.AddedItems.Count > 0)
      {
        ExpandCollapseMedia(e.AddedItems[0] as FrameworkElement);
      }
    }

    void expandStoryboard_Completed(object sender, EventArgs e)
    {
      Canvas.SetLeft(Display, Canvas.GetLeft(ThumbDisplay1));
      Display.Width = ThumbDisplay1.ActualWidth * _expansionFactor;
      Display.Fill = ThumbDisplay1.Fill;
      Display.Opacity = 1;
      if (_fadeStoryboard != null)
      {
        _fadeStoryboard.Stop();
        _fadeStoryboard = null;
      }
      ThumbDisplay1.Opacity = 0;
      ThumbDisplay1.Fill = null;
      (sender as Storyboard).Stop();
    }

    void ExpandCollapseMedia(FrameworkElement media)
    {
      // If needed, pause movie and fade display
      if (_expandedMedia != null)
      {
        if (_expandedMedia is MediaElement)
        {
          (_expandedMedia as MediaElement).Pause();
        }
      
        FrameworkElement currentMedia = _expandedMedia;
        _expandedMedia = null;
      
        _fadeStoryboard = new Storyboard();
        _fadeStoryboard.Completed += new EventHandler(_fadeStoryboard_Completed);
      
        DoubleAnimation anim = new DoubleAnimation();
        Storyboard.SetTarget(anim, Display);
        Storyboard.SetTargetProperty(anim, new PropertyPath("Opacity"));
        anim.Duration = new Duration(TimeSpan.FromSeconds(0.9));
        anim.To = 0;
        _fadeStoryboard.Children.Add(anim);
      
        _fadeStoryboard.Begin();
      
        if (media == currentMedia)
        {
          return;
        }
      }

      // Set size and position of the displayed media
      double sizeFactor
        = media.ActualWidth / media.ActualHeight;
      double newHeight = DisplayBackground.ActualHeight;
      double newWidth = newHeight * sizeFactor;
      double sideMargin
        = (DisplayBackground.ActualWidth - newWidth) / 2;
      _expansionFactor
        = DisplayBackground.ActualHeight / media.ActualHeight;

      if (sideMargin < 0)
      {
        newWidth = DisplayBackground.ActualWidth;
        newHeight = newWidth / sizeFactor;
        sideMargin = 0;
        _expansionFactor
          = DisplayBackground.ActualWidth / media.ActualWidth;
      }

      // Set the placeholder's size and calculate its proportions
      ThumbDisplay1.Width = media.ActualWidth;
      ThumbDisplay1.Height = media.ActualHeight;

      // Getting the coordinates of the clicked thumbnail
      GeneralTransform gtStart =
        media.TransformToVisual(AnimationCanvas);
      Point offsetStart = gtStart.Transform(new Point(0, 0));
      ThumbDisplay1.SetValue(Canvas.LeftProperty, offsetStart.X);
      ThumbDisplay1.SetValue(Canvas.TopProperty, offsetStart.Y);

      // Getting the coordinates of the DisplayBackground rectangle
      GeneralTransform gtDisplay =
        DisplayBackground.TransformToVisual(AnimationCanvas);
      Point offsetDisplay = gtDisplay.Transform(new Point(0, 0));

      // Get the storyboard and set the animations' parameters
      Storyboard expandStoryboard
        = TopUserControl.Resources["ExpandStoryboard"] as Storyboard;

      foreach (Timeline child in expandStoryboard.Children)
      {
        string property = Storyboard.GetTargetProperty(child).Path;
        if (property.IndexOf("TranslateTransform.X") > -1)
        {
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[0].Value = 0;
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[1].Value
            = offsetDisplay.X - offsetStart.X + sideMargin;
        }
        if (property.IndexOf("TranslateTransform.Y") > -1)
        {
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[0].Value = 0;
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[1].Value
            = offsetDisplay.Y - offsetStart.Y;
        }
        if (property.IndexOf("ScaleTransform.ScaleX") > -1)
        {
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[0].Value = 1;
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[1].Value
            = _expansionFactor;
        }
        if (property.IndexOf("ScaleTransform.ScaleY") > -1)
        {
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[0].Value = 1;
          (child as DoubleAnimationUsingKeyFrames).KeyFrames[1].Value
            = _expansionFactor;
        }
      }
      
      // Set the Display and start movie (if needed)
      if (media is MediaElement)
      {
        VideoBrush brush = new VideoBrush();
        brush.SetSource(media as MediaElement);
        (media as MediaElement).Play();
        ThumbDisplay1.Fill = brush;
      }
      if (media is Image)
      {
        ImageBrush brush = new ImageBrush();
        BitmapImage source
          = (media as Image).Source as BitmapImage;
        brush.ImageSource
          = new BitmapImage(source.UriSource);
        ThumbDisplay1.Fill = brush;
      }

      _expandedMedia = media;
      //Start the Storyboard
      expandStoryboard.Begin();
    }

    void _fadeStoryboard_Completed(object sender, EventArgs e)
    {
      Display.Fill = null;
      (sender as Storyboard).Stop();
    }

    [ScriptableMember]
    public string AddUser(string name, string password)
    {
      try
      {
        User newUser = new User();
        newUser.Name = name;
        newUser.Password = password;
        newUser.SetLastVisit();
        _dataFile.AddUser(newUser);
      }
      catch (Exception ex)
      {
        return ex.Message;
      }

      LastVisitTextBlock.Text = "(" + name + ", this is your first visit)";
      return null; // success
    }

    [ScriptableMember]
    public string ValidateUser(string name, string password)
    {
      User inputUser = new User();
      inputUser.Name = name;
      inputUser.Password = password;
      if (_dataFile.ValidateUser(inputUser))
      {
        User savedUser = _dataFile.GetUser(name);
        LastVisitTextBlock.Text = "(" + name + ", your last visit was: "
          + savedUser.LastVisit.Value.ToShortDateString()
          + " " + savedUser.LastVisit.Value.ToLongTimeString() + ")";
        savedUser.SetLastVisit();
        _dataFile.SaveUsers();
        return null; // success
      }
      else
      {
        return "Invalid user/password combination";
      }
    }

    public void OnLoginClicked(EventArgs e)
    {
      if (LoginClicked != null)
      {
        LoginClicked(this, e);
      }
    }

    private void LoginLink_Click(object sender, RoutedEventArgs e)
    {
      EventArgs args = new EventArgs();
      OnLoginClicked(args);
    }
  }
}