﻿using System;
using System.Net;
using System.IO;
using System.IO.IsolatedStorage;
using System.Collections.Generic;

namespace UserManagement
{
  public class DataFile
  {
    private const string DATA_FILE_NAME = @"c:\temp\ThumbnailsData.3.txt";

    private Dictionary<string, User> _users;

    public void LoadUsers()
    {
      _users = new Dictionary<string, User>();
    
      if (File.Exists(DATA_FILE_NAME))
      {
        using (StreamReader reader = File.OpenText(DATA_FILE_NAME))
        {
          string line;
          do
          {
            line = reader.ReadLine();
            if (line != null)
            {
              User newUser = new User(line);
              _users.Add(newUser.Name, newUser);
            }
          }
          while (line != null);
        }
      }
    }

    public void SaveUsers()
    {
      using (StreamWriter writer = File.CreateText(DATA_FILE_NAME))
      {
        foreach (User user in _users.Values)
        {
          writer.WriteLine(user.ToString());
        }
      }
    }

    public void AddUser(User newUser)
    {
      if (newUser.Name == null
        || newUser.Name.Length == 0)
      {
        throw new Exception("Name may not be empty");
      }
      if (_users.ContainsKey(newUser.Name))
      {
        throw new Exception("This name already exists");
      }
      _users.Add(newUser.Name, newUser);
      SaveUsers();
    }

    public bool ValidateUser(User inputUser)
    {
      if (inputUser == null)
      {
        // Refuse to validate null users
        return false;
      }
      User existingUser = GetUser(inputUser.Name);
      return (existingUser == inputUser);
    }

    public User GetUser(string name)
    {
      if (_users.ContainsKey(name))
      {
        return _users[name];
      }
      else
      {
        return null;
      }
    }
  }
}
